==> tecfy_corev2_package-1.1.30.tar.gz: lib/models/entity/property_dropdown.model.dart <==
part of tecfy_corev2_client_package;

class PropertyDropdown extends Property {
  late String entityName;

  Property? entityDisplayProp;

  /// <summary>
  /// used to get the list using custom EndPoint like "Item/GetItemInfo?brand={brandId}&model={modelId}&category={itemCategoryId}&year={year}"
  /// </summary>
  late String dropdownUrl;

  late bool isFilter;

  /// <summary>
  /// where value of FilterField is null of embty then the value of FilterValueField must refer to Array proerty of same dropdwn Entity
  /// </summary>
  late String filterField;
  late String filterQuery;

  /// <summary>
  /// where value of FilterField is null of embty then the value of FilterValueField must refer to Array proerty of same dropdwn Entity
  /// </summary>
  late String filterValueField;

  List<Map<String, String>>? options;

  /// <summary>
  /// Fileds that in main entity and dropdown entity with the same name, that you need to update the value of the main entity with the value in dropdown entity
  /// For example: when select item the UOM_ID of the invoice will be overwrten
  /// Can add multi properties seperated by comma
  /// </summary>
  Map<String, String>? overwritePropertiesOnSelect;

  PropertyDropdown(
      {required String dbName,
      required String nameAr,
      required String nameEn,
      bool isRequired = false,
      bool readOnly = false,
      required this.entityName,
      this.overwritePropertiesOnSelect,
      this.isFilter = false,
      this.filterField = '',
      this.filterQuery = '',
      this.filterValueField = ''})
      : super(
            dbName: dbName,
            nameAr: nameAr,
            nameEn: nameEn,
            readOnly: readOnly,
            propertyType: PropertyTypes.Dropdown,
            valueType: ValueTypes.Text,
            isRequired: isRequired) {
    dropdownUrl = '';
    bindOptions();
  }

  PropertyDropdown.fromMap(Map<String, dynamic> row) : super.fromMap(row) {
    entityName = row['entityName'] ?? '';
    dropdownUrl = row['dropdownUrl'] ?? '';
    isFilter = row['isFilter'] ?? false;
    filterField = lowerFirst(row['filterField'] ?? '') ?? '';
    filterQuery = row['filterQuery'] ?? '';

    overwritePropertiesOnSelect = {};

    if (row['overwritePropertiesOnSelect'] != null) {
      for (var k in row['overwritePropertiesOnSelect'].toString().split(',')) {
        var s = k.split(':');
        overwritePropertiesOnSelect![lowerFirst(s.first)!] =
            lowerFirst(s.last)!;
      }
    }

    dbName = lowerFirst(row['dbName']) ?? '';

    filterValueField = lowerFirst(row['filterValueField'] ?? '') ?? '';
    if (entityName.isEmpty && filterValueField.isNotEmpty) {
      filterValueField = row['filterValueField'];
    }
    bindOptions();
  }

  void bindOptions() {
    if (entityName.isEmpty && filterValueField.isNotEmpty) {
      options = filterValueField.split(',').map<Map<String, String>>((e) {
        var s = e.split(':');
        return {
          'id': s.first,
          'nameEn': s.first,
          'nameAr': s.last,
        };
      }).toList();
    }
  }

  @override
  OperationResult isValid(data) {
    if (!super.isValid(data).success) return super.isValid(data);

    return OperationResult(success: true);
  }

  /// get the value and bind tp display widget
  Widget getDisplayWidget(Map<String, dynamic> data, {TextStyle? textStyle}) {
    return Text(getDisplayText(data), style: textStyle);
  }

  /// get the value in text
  String getDisplayText(Map<String, dynamic> data) {
    if (entityName.isEmpty && filterValueField.isNotEmpty) {
      return data[dbName] ?? '';
    }
    // if (this.dbNameEntity.isEmpty &&
    //     this.dbName.isNotEmpty &&
    //     this.dbName.length > 2) {
    //   dbNameEntity = this.dbName.substring(0, this.dbName.length - 2);
    // }
    if (entityDisplayProp == null && entityName.isNotEmpty) {
      entityDisplayProp = EntityService.getEntity(entityName)?.displayField;
    }
    return data[this.dbName] is String
        ? data[this.dbName]
        : data[this.dbName] == null
            ? '-'
            : '${data[this.dbName][entityDisplayProp?.dbName ?? 'name']}';
  }

  Future<bool> getDefaultValue(Map<String, dynamic> data) async {
    //if(data[dbName] != null) return false;
    if (entityName.isEmpty && filterValueField.isNotEmpty) {
      data[dbName] = data[dbName] ?? defaultString;
      return true;
    }

    var dbNameParent = dbName + '______Parent';
    try {
      //'==================== set default value of dropdown =============== $dbName   $dbNameEntity');
      // Get the default value and set it
      if (defaultString?.isNotEmpty ?? false) {
        if (int.tryParse(defaultString ?? '') != null) {
          // TODO: fix after you make Id string
          if (data[dbName] == null || data[dbName] == 0) {
            data[dbName] = int.parse(defaultString ?? '');
            return true;
          }
        } else {
          if (defaultString?.contains('.') ?? false) {
            var parentPropName = defaultString?.split('.')[0] ?? '';
            var childPropName = defaultString?.split('.')[1] ?? '';

            var parentData = data[lowerFirst(parentPropName)];
            if (data[dbNameParent] != null &&
                data[dbNameParent]['_id'] == parentData['_id']) return false;
            data[dbNameParent] = parentData;
            var childData = parentData[lowerFirst(childPropName)];
            if (childData is int) {
              if (data[dbName] != childData) {
                data[dbName] = childData;
              }
            } else {
              // TODO: hadel this case
            }
          } else {
            // TODO: hadel this case
          }
        }
      }
    } catch (err, t) {
      print(
          'XXXXXXXXXXXXxxxxxxxxxxxx ERROR  getDefaultValue for Dropdown $dbName: $err');
      print(t);
    }
    return false;
  }
}

==> tecfy_corev2_package-1.1.35.tar.gz: lib/models/entity/property_dropdown.model.dart <==
part of tecfy_corev2_client_package;

class PropertyDropdown extends Property {
  late String entityName;

  Property? entityDisplayProp;

  /// <summary>
  /// used to get the list using custom EndPoint like "Item/GetItemInfo?brand={brandId}&model={modelId}&category={itemCategoryId}&year={year}"
  /// </summary>
  late String dropdownUrl;

  late bool isFilter;

  /// <summary>
  /// where value of FilterField is null of embty then the value of FilterValueField must refer to Array proerty of same dropdwn Entity
  /// </summary>
  late String filterField;
  late String filterQuery;

  /// <summary>
  /// where value of FilterField is null of embty then the value of FilterValueField must refer to Array proerty of same dropdwn Entity
  /// </summary>
  late String filterValueField;

  List<Map<String, String>>? options;

  /// <summary>
  /// Fileds that in main entity and dropdown entity with the same name, that you need to update the value of the main entity with the value in dropdown entity
  /// For example: when select item the UOM_ID of the invoice will be overwrten
  /// Can add multi properties seperated by comma
  /// </summary>
  Map<String, String>? overwritePropertiesOnSelect;

  PropertyDropdown(
      {required String dbName,
      required String nameAr,
      required String nameEn,
      bool isRequired = false,
      bool readOnly = false,
      required this.entityName,
      this.overwritePropertiesOnSelect,
      this.isFilter = false,
      this.filterField = '',
      this.filterQuery = '',
      this.filterValueField = ''})
      : super(
            dbName: dbName,
            nameAr: nameAr,
            nameEn: nameEn,
            readOnly: readOnly,
            propertyType: PropertyTypes.Dropdown,
            valueType: ValueTypes.Text,
            isRequired: isRequired) {
    dropdownUrl = '';
    bindOptions();
  }

  PropertyDropdown.fromMap(Map<String, dynamic> row) : super.fromMap(row) {
    entityName = row['entityName'] ?? '';
    dropdownUrl = row['dropdownUrl'] ?? '';
    isFilter = row['isFilter'] ?? false;
    filterField = lowerFirst(row['filterField'] ?? '') ?? '';
    filterQuery = row['filterQuery'] ?? '';

    overwritePropertiesOnSelect = {};

    if (row['overwritePropertiesOnSelect'] != null) {
      for (var k in row['overwritePropertiesOnSelect'].toString().split(',')) {
        var s = k.split(':');
        overwritePropertiesOnSelect![lowerFirst(s.first)!] =
            lowerFirst(s.last)!;
      }
    }

    dbName = lowerFirst(row['dbName']) ?? '';

    filterValueField = lowerFirst(row['filterValueField'] ?? '') ?? '';
    if (entityName.isEmpty && filterValueField.isNotEmpty) {
      filterValueField = row['filterValueField'];
    }
    bindOptions();
  }

  void bindOptions() {
    if (entityName.isEmpty && filterValueField.isNotEmpty) {
      options = filterValueField.split(',').map<Map<String, String>>((e) {
        var s = e.split(':');
        return {
          'id': s.first,
          'nameEn': s.first,
          'nameAr': s.last,
        };
      }).toList();
    }
  }

  @override
  OperationResult isValid(data) {
    if (!super.isValid(data).success) return super.isValid(data);

    return OperationResult(success: true);
  }

  /// get the value and bind tp display widget
  Widget getDisplayWidget(Map<String, dynamic> data, {TextStyle? textStyle}) {
    return Text(getDisplayText(data), style: textStyle);
  }

  /// get the value in text
  String getDisplayText(Map<String, dynamic> data) {
    var value = MapService.getValue(data, dbName);
    if (dbName.contains('.')) {
      if (value != null) {
        return value.toString();
      }
    } else {
      if (entityName.isEmpty && filterValueField.isNotEmpty) {
        return value ?? '';
      }

      if (entityDisplayProp == null && entityName.isNotEmpty) {
        entityDisplayProp = EntityService.getEntity(entityName)?.displayField;
      }
      if (value is Map && value[entityDisplayProp?.dbName] is Map) {
        return value[entityDisplayProp?.dbName]
                [LocalConfigService.languagePrefix.toLowerCase()] ??
            '';
      } else if (value is Map) {
        return value[entityDisplayProp?.dbName] ?? '';
      }
      return value is String
          ? value
          : value == null
              ? '-'
              : '${value[entityDisplayProp?.dbName ?? 'name']}';
    }
    return '';

    // if (entityName.isEmpty && filterValueField.isNotEmpty) {
    //   return MapService.getValue(data, dbName.split('.'),0) ?? '';
    // }
    // if (dbName.isEmpty &&
    //     this.dbName.isNotEmpty &&
    //     this.dbName.length > 2) {
    //   dbName = this.dbName.substring(0, this.dbName.length - 2);
    // }
    // if (entityDisplayProp == null && entityName.isNotEmpty) {
    //   entityDisplayProp = EntityService.getEntity(entityName)?.displayField;
    // }
    // return MapService.getValue(data, dbName.split('.'),0) is String
    //     ? MapService.getValue(data, dbName.split('.'),0)
    //     :MapService.getValue(data, dbName.split('.'),0) == null
    //     ? '-'
    //     : '${MapService.getValue(data, dbName.split('.'),0)[entityDisplayProp?.dbName ?? 'name']}';
  }

  Future<bool> getDefaultValue(Map<String, dynamic> data) async {
    //if(data[dbName] != null) return false;
    if (entityName.isEmpty && filterValueField.isNotEmpty) {
      // data[dbName] = data[dbName] ?? defaultString;
      MapService.updateValue(
          data, dbName, MapService.getValue(data, dbName) ?? defaultString);
      return true;
    }

    var dbNameParent = dbName + '______Parent';
    try {
      //'==================== set default value of dropdown =============== $dbName   $dbNameEntity');
      // Get the default value and set it
      if (defaultString?.isNotEmpty ?? false) {
        if (int.tryParse(defaultString ?? '') != null) {
          // TODO: fix after you make Id string
          if (MapService.getValue(data, dbName) == null ||
              MapService.getValue(data, dbName) == 0) {
            // data[dbName] = int.parse(defaultString ?? '');
            MapService.updateValue(data, dbName, defaultString ?? '');
            return true;
          }
        } else {
          if (defaultString?.contains('.') ?? false) {
            var parentPropName = defaultString?.split('.')[0] ?? '';
            var childPropName = defaultString?.split('.')[1] ?? '';

            var parentData = data[lowerFirst(parentPropName)];
            if (data[dbNameParent] != null &&
                data[dbNameParent]['_id'] == parentData['_id']) return false;
            data[dbNameParent] = parentData;
            var childData = parentData[lowerFirst(childPropName)];
            if (childData is int) {
              if (MapService.getValue(data, dbName) != childData) {
                // data[dbName] = childData;
                MapService.updateValue(data, dbName, childData);
              }
            } else {
              // TODO: hadel this case
            }
          } else {
            // TODO: hadel this case
          }
        }
      }
    } catch (err, t) {
      print(
          'XXXXXXXXXXXXxxxxxxxxxxxx ERROR  getDefaultValue for Dropdown $dbName: $err');
      print(t);
    }
    return false;
  }
}
